import {
    EncryptionAlgorithm,
    MatrixClient,
    MessageEvent,
    RustSdkCryptoStorageProvider,
    SimpleFsStorageProvider,
} from "matrix-bot-sdk";

import { StoreType } from "@matrix-org/matrix-sdk-crypto-nodejs";

import {
    convertMsgType,
    logError,
    logInfo,
    readConfig,
    uploadMediaFromEvent
} from './utils.js';

import {
    appendUserLikes,
    appendUserPictures,
    checkForMutualLike,
    eraseUser,
    eraseUserLikes,
    eraseUserMedia,
    getAmountOfUserPictures,
    getCurrentUserAction,
    getUnreadMessages,
    getUserCurrentlyViewingProfile,
    insertMessageIntoDB,
    markMessageAsRead,
    setUserState,
    uploadMediaAsMessage,
    setUserLanguage,
    getUserLanguage
} from './db.js';

import { processRequest, showRandomProfileToUser, showNewLikes } from "./interactions.js";

import { db } from "./db.js";

import fs from "fs";

import { I18n } from "i18n-js";
const i18n = new I18n({
    en: JSON.parse(fs.readFileSync("./translations/en.json")),
    ru: JSON.parse(fs.readFileSync("./translations/ru.json"))
});
i18n.defaultLocale = "en";

const config = readConfig();

const homeserverUrl = config.homeserverURL;
const accessToken = config.token;
const maxAmountOfPhotoesPerUser = config.maxAmountOfPhotoesPerUser;

const crypto = new RustSdkCryptoStorageProvider("./bot_data/encryption_bot_sled", StoreType.Sled);
const storage = new SimpleFsStorageProvider("./bot_data/bot.json");

const client = new MatrixClient(homeserverUrl, accessToken, storage, crypto);

client.on("room.message", async (roomId, event) => {
    try {
        if (event.content?.msgtype !== 'm.text' && event.content?.msgtype !== 'm.image' && event.content?.msgtype !== 'm.video') return;
        if (event.sender === await client.getUserId()) return;

        let current_action = await getCurrentUserAction(roomId);
        let answer = event.content.body;
        let msgtype = event.content.msgtype

        let preferredLanguage = await getUserLanguage(roomId);
        if (!preferredLanguage) preferredLanguage = i18n.defaultLocale;
        i18n.locale = preferredLanguage;

        switch (current_action) {
            case "wait_start":
                let a = answer.split(" ")
                if (a[0] !== "!start") return;
                if (a[1] !== "ru" && a[1] !== "en") return;
                await setUserLanguage(roomId, a[1]);
                await setUserState(roomId, "country");
                await client.sendText(roomId, i18n.t(["setup", "country"]));
                
                break;
            case "country":
                await processRequest(client, roomId, current_action, answer, 'city');
                break;
            case "city":
                await processRequest(client, roomId, current_action, answer, 'name');
                break;
            case "name":
                await processRequest(client, roomId, current_action, answer, 'age');
                break;
            case "age":
                answer = parseInt(answer)
                if (!answer) {
                    await client.sendText(roomId, i18n.t(["general", "age"]));
                    return;
                }

                if (answer < 14) {
                    await client.sendText(roomId, i18n.t(["errors", "tooyoung"]));

                    await client.leaveRoom(roomId);
                    await eraseUser(roomId);
                    return;
                }
                await processRequest(client, roomId, current_action, answer, 'sex');
                break;
            case "sex":
                answer = answer.toLowerCase().trim();
                if (answer.toLowerCase() != "male" && answer.toLowerCase() != "female") {
                    await client.sendText(roomId, i18n.t(["errors", "twosexes"]));

                    return;
                }
                await processRequest(client, roomId, current_action, answer[0], 'interest');
                break;
            case "interest":
                answer = answer.toLowerCase().trim();
                if (answer != "male" && answer != "female" && answer != "both") {
                    await client.sendText(roomId, i18n.t(["errors", "didntunderstand"]));

                    return;
                }
                await processRequest(client, roomId, current_action, answer[0], 'description');
                break;
            case "description":
                await processRequest(client, roomId, current_action, answer, 'pictures');
                break;
            case "pictures":
                if (event.content?.msgtype !== 'm.image' && event.content?.msgtype !== 'm.video') {
                    await client.sendText(roomId, i18n.t(["setup", "done"]));

                    await setUserState(roomId, 'view_profiles');
                    await showRandomProfileToUser(client, roomId);
                } else {
                    let pictures_count = parseInt(await getAmountOfUserPictures(roomId));
                    if (pictures_count >= maxAmountOfPhotoesPerUser) {
                        await client.sendText(roomId, i18n.t(["errors", "toomuch"]));

                        await setUserState(roomId, 'view_profiles');
                        await showRandomProfileToUser(client, roomId);
                    } else {
                        let mxc = await uploadMediaFromEvent(client, event);
                        let type = convertMsgType(msgtype);
                        await appendUserPictures(roomId, mxc, type);
                        let pictures_count = await getAmountOfUserPictures(roomId);
                        if (pictures_count < maxAmountOfPhotoesPerUser) {
                            await client.sendText(roomId, i18n.t(["setup", "more"], { amount: String(maxAmountOfPhotoesPerUser - pictures_count) }));

                        } else {
                            await client.sendText(roomId, i18n.t(["setup", "enough"]));

                            await client.sendText(roomId, i18n.t(["setup", "done"]));

                            await setUserState(roomId, 'view_profiles');
                            await showRandomProfileToUser(client, roomId);
                        }
                    }
                }
                break;
            case "view_profiles":
                if (answer == '👍️' || answer == '❤️' || answer == '1') {
                    let currently_viewing = await getUserCurrentlyViewingProfile(roomId);
                    await appendUserLikes(roomId, currently_viewing);
                    let value = await checkForMutualLike(roomId, currently_viewing);
                    if (value) {
                        await client.sendText(roomId, i18n.t(["general", "newlikes"]));

                        await client.sendText(roomId, i18n.t(["general", "newlikes"]));

                    }
                } else if (answer == '💌' || answer == '3') {
                    await setUserState(roomId, 'send_message');
                    await client.sendText(roomId, i18n.t(["general", "message"]));

                    return;
                } else if (answer == '🏠️' || answer == '4') {
                    await client.sendText(roomId, messages.general.menu);
                    await setUserState(roomId, 'menu');
                    return;
                }

                await showRandomProfileToUser(client, roomId);
                break;
            case 'send_message':
                let recipient = await getUserCurrentlyViewingProfile(roomId);
                let content;
                if (msgtype == "m.image" || msgtype == "m.video") {
                    let mxc = await uploadMediaFromEvent(client, event);
                    await uploadMediaAsMessage(roomId, msgtype, mxc);
                    content = mxc;
                } else {
                    content = answer;
                }
                if (await insertMessageIntoDB(roomId, recipient, msgtype, content)) {
                    await client.sendText(roomId, i18n.t(["general", "newmessage"]));

                    await client.sendText(roomId, i18n.t(["general", "messagesent"]));

                } else {
                    await client.sendText(roomId, i18n.t(["general", "alreadymessaged"]));

                }
                await setUserState(roomId, 'view_profiles');
                await showRandomProfileToUser(client, roomId);
                break;
            case 'menu':
                switch (answer) {
                    case '1':
                        await setUserState(roomId, 'view_profiles');
                        await showRandomProfileToUser(client, roomId);
                        break;
                    case '2':
                        await showNewLikes(client, roomId);
                        await client.sendText(roomId, i18n.t(["general", "menu"]));

                        break;
                    case '3':
                        let unreadMessages = await getUnreadMessages(roomId);
                        if (!unreadMessages || unreadMessages.length == 0) {
                            await client.sendText(roomId, i18n.t(["general", "nonewmessages"]));

                            return;
                        }
                        await client.sendText(roomId, i18n.t(["general", "msg"]));

                        for (let message of unreadMessages) {
                            await client.sendText(roomId, i18n.t(["general", "showmessage"], { user: message.mx_id }));

                            if (message.type == "t") {
                                await client.sendText(roomId, message.content)
                            } else if (message.type == "p" || message.type == "v") {
                                let msgtype;

                                if (message.type == "p") {
                                    msgtype = "m.image"
                                } else if (message.type == "v") {
                                    msgtype = "m.video";
                                }

                                await client.sendMessage(roomId, {
                                    msgtype: msgtype,
                                    body: "Message",
                                    url: message.content
                                });
                            }
                            await markMessageAsRead(message.sender, roomId);
                        }
                        await client.sendText(roomId, i18n.t(["general", "menu"]));
                        break;
                    default:
                        await client.sendText(roomId, i18n.t(["general", "didntunderstand"]));

                        await client.sendText(roomId, i18n.t(["general", "menu"]));

                        break;
                }
                break;
            default:
                await client.sendText(roomId, i18n.t(["general", "didntunderstand"]));

                return;
        }
    } catch (e) {
        logError(e);
    }
});

client.on("room.event", async (roomId, event) => {
    if (event.type === "m.room.member" && event.content?.membership === "leave") {
        await eraseUser(roomId);
        await eraseUserLikes(roomId);
        await eraseUserMedia(roomId);
        logInfo(`Bot has left a room with ID  ${roomId}`)
        client.leaveRoom(roomId);
    }
})

client.on("room.invite", async (roomId, event) => {
    try {
        await client.joinRoom(roomId);

        let members = await client.getAllRoomMembers(roomId)
        let isDM = members.length == 2 ? true : false;

        if (!isDM) {
            await client.sendText(roomId, i18n.t(["errors", "notadm"]));

            await client.leaveRoom(roomId)
        }

        logInfo(`Bot has joined a room with ID ${roomId}`)

        let mx_id = event.sender

        let is_profile_exists = (await db.query("SELECT * FROM users WHERE room_id = $1", [roomId])).rowCount > 0;

        if (!is_profile_exists) {
            await db.query("INSERT INTO users(mx_id, room_id, current_action) VALUES ($1, $2, $3)", [mx_id, roomId, "wait_start"])

            i18n.locale = "en";
            await client.sendText(roomId, i18n.t(["general", "welcome"]));
            i18n.locale = "ru";
            await client.sendText(roomId, i18n.t(["general", "welcome"]));

        } else {
            setUserState(roomId, 'view_profiles')
        }
    } catch (e) {
        logError(e)
    }

});

client.start().then(() => logInfo("Bot started!"));